==> dataspree_inference-2.7.1-cp310-cp310-win_amd64.whl: dataspree/inference/core/simple_plugin.py <==
import logging

import copy
from abc import ABC, abstractmethod
from typing import Any, Optional, List, Set, Tuple, Union, Iterable

from dataspree.inference.core.consumer import Consumer
from dataspree.inference.core.item import Item
from dataspree.inference.core.plugin import Plugin
from dataspree.inference.core.producer import Producer, Property
from dataspree.inference.core.exceptions import ImplementationError, StopTimeout

logger = logging.getLogger(__name__)


class SimplePlugin(Plugin, ABC):
    def __init__(self, name: str, properties: Optional[List[Property]] = None, input_queue_size=1,
                 producers: Optional[List[Union[Tuple[str, List[Property]], str]]] = None) -> None:
        Plugin.__init__(self, name)

        if producers is None:
            producers = [(name, properties or [])]

        for producer_i, producer in enumerate(producers):

            current_name: str
            current_properties: List[Property]
            if isinstance(producer, str):
                current_name, current_properties = producer, copy.copy(properties or [])
            else:
                if len(producer) != 2 or not isinstance(producer[0], str):
                    raise ImplementationError(f'Invalid argument for producers: Producer {producer_i} is neither '
                                              f'Tuple[str, List[Properties]] nor str but "{producer}".'
                                              f'Not instantiating ill-formed class {type(self).__name__}')
                current_name, current_properties = producer

            self.add_producer(Producer(current_name, current_properties))

        self.producers_init: List[str] = list(self.producers.keys())

        self.consumer: Consumer = Consumer(name, input_queue_size, signal_handler=self)
        self.add_consumer(self.consumer)

    @abstractmethod
    def process_item(self, item: Item) -> Union[Optional[Item], Tuple[Iterable[str], Item]]:
        """Process a single item and return it"""

    def on_output_dropped(self, item: Item, consumers: Set[str]) -> None:
        """
        Evoked whenever output is dropped at at least one consumer.

        :param item:       The item that was dropped
        :param consumers:  The consumers that did not take the item.
        """
        pass

    def on_stopped(self, exception: Optional[BaseException] = None) -> Optional[bool]:
        """
        Callback that is evoked when run terminates. Can be implemented by a derived class.

        :param exception:  Exception that was raised and terminates this the run function or None if the run
                           exited gracefully.

        :return:           True if the exception is not to to be raised.
                           Otherwise (i.e., if the return value is None, False or of any type except bool),
                           the exception is thrown immediately after on_stopped exits.
        """
        return None

    def forward_item(self, item: Item, *, producers: Optional[List[Producer]] = None, **kwargs):
        if producers is None:
            producers = list(self.producers.values())

        # Forward the processed result if any.
        if item is not None:
            for producer in producers:
                dropped, _successful = producer.send(item, **kwargs)

                if len(dropped):
                    self.on_output_dropped(item, dropped)

    def run(self) -> None:
        ex: Optional[BaseException] = None

        try:
            while self.running:
                item: Optional[Item] = self.consumer.receive()

                if item is not None:

                    producer_name: Optional[str]
                    result: Union[Optional[Item], Tuple[Iterable[str], Item]] = self.process_item(item)
                    if result is not None:

                        processed_item: Item
                        producers: List[Producer]
                        if isinstance(result, dict):
                            processed_item = Item(result)
                            producers = [self.producers[k] for k in self.producers_init if
                                         self.producers.get(k) is not None]
                            if not len(producers):
                                producers = list(self.producers.values())

                        else:
                            producer_names: Iterable[str]
                            producer_names, processed_item = result

                            invalid_producers: List[str] = [producer_name for producer_name in producer_names
                                                            if producer_name not in self.producers]
                            if any(invalid_producers):
                                logger.warning(f'Invalid producer: {", ".join(invalid_producers)}')
                                producer_names = [producer_name for producer_name in producer_names
                                                  if producer_name in self.producers]
                            producers = [self.producers[producer_name] for producer_name in producer_names]

                        if not len(producers):
                            logger.warning('Sending an item to an empty set of producers.')

                        if processed_item is not None:
                            self.forward_item(processed_item, producers=producers)

        except BaseException as e:
            ex = e

        finally:
            self.stop()
            noexcept_exit: Any = self.on_stopped(ex)

            if ex is not None:
                if noexcept_exit is not None and type(noexcept_exit) != bool:
                    logger.warning(f'Return value of `on_stopped` implementation is not bool: {noexcept_exit}. '
                                   f'Raising exception independent of true value of {noexcept_exit}.')

                # If `ex` is not implementing Exception raise either way, independently on whether
                # on_stopped told us to discard the exception
                if not isinstance(ex, BaseException):
                    logger.warning(f'Raised {ex} of type {type(ex)}. Wrapping.')
                    raise Exception(f'{ex}')

                # If on_stopped wants us to raise the exception, we raise.
                if not noexcept_exit:
                    raise ex from None

                # If on_stopped is not implemented or does not override the default, raise whenever the exception
                # is not an instance of StopTimeout. StopTimeouts are only reported.
                if type(noexcept_exit) != bool:
                    if isinstance(ex, StopTimeout):
                        logger.info(f'Node {self.name} stopped ungracefully with a stop timeout.')
                    else:
                        raise ex from None

==> dataspree_inference-2.7.3-cp310-cp310-win_amd64.whl: dataspree/inference/core/simple_plugin.py <==
import logging

import copy
from abc import ABC, abstractmethod
from typing import Any, Optional, List, Set, Tuple, Union, Iterable

from dataspree.inference.core.consumer import Consumer
from dataspree.inference.core.item import Item
from dataspree.inference.core.plugin import Plugin
from dataspree.inference.core.producer import Producer, Property
from dataspree.inference.core.exceptions import ImplementationError, StopTimeout

logger = logging.getLogger(__name__)


class SimplePlugin(Plugin, ABC):
    def __init__(self, name: Optional[str], properties: Optional[List[Property]] = None, input_queue_size=1,
                 producers: Optional[List[Union[Tuple[str, List[Property]], str]]] = None) -> None:
        Plugin.__init__(self, name)

        if producers is None:
            if name is None:
                producers = []
            else:
                producers = [(name, properties or [])]

        for producer_i, producer in enumerate(producers):

            current_name: str
            current_properties: List[Property]
            if isinstance(producer, str):
                current_name, current_properties = producer, copy.copy(properties or [])
            else:
                if len(producer) != 2 or not isinstance(producer[0], str):
                    raise ImplementationError(f'Invalid argument for producers: Producer {producer_i} is neither '
                                              f'Tuple[str, List[Properties]] nor str but "{producer}".'
                                              f'Not instantiating ill-formed class {type(self).__name__}')
                current_name, current_properties = producer

            self.add_producer(Producer(current_name, current_properties))

        self.producers_init: List[str] = list(self.producers.keys())

        self.consumer: Consumer = Consumer(name, input_queue_size, signal_handler=self)
        self.add_consumer(self.consumer)

    @abstractmethod
    def process_item(self, item: Item) -> Union[Optional[Item], Tuple[Iterable[str], Item]]:
        """Process a single item and return it"""

    def on_output_dropped(self, item: Item, consumers: Set[str]) -> None:
        """
        Evoked whenever output is dropped at at least one consumer.

        :param item:       The item that was dropped
        :param consumers:  The consumers that did not take the item.
        """
        pass

    def on_stopped(self, exception: Optional[BaseException] = None) -> Optional[bool]:
        """
        Callback that is evoked when run terminates. Can be implemented by a derived class.

        :param exception:  Exception that was raised and terminates this the run function or None if the run
                           exited gracefully.

        :return:           True if the exception is not to to be raised.
                           Otherwise (i.e., if the return value is None, False or of any type except bool),
                           the exception is thrown immediately after on_stopped exits.
        """
        return None

    def forward_item(self, item: Item, *, producers: Optional[List[Producer]] = None, **kwargs):
        if producers is None:
            producers = list(self.producers.values())

        # Forward the processed result if any.
        if item is not None:
            for producer in producers:
                dropped, _successful = producer.send(item, **kwargs)

                if len(dropped):
                    self.on_output_dropped(item, dropped)

    def run(self) -> None:
        ex: Optional[BaseException] = None

        try:
            while self.running:
                item: Optional[Item] = self.consumer.receive()

                if item is not None:

                    producer_name: Optional[str]
                    result: Union[Optional[Item], Tuple[Iterable[str], Item]] = self.process_item(item)
                    if result is not None:

                        processed_item: Item
                        producers: List[Producer]
                        if isinstance(result, dict):
                            processed_item = Item(result)
                            producers = [self.producers[k] for k in self.producers_init if
                                         self.producers.get(k) is not None]
                            if not len(producers):
                                producers = list(self.producers.values())

                        else:
                            producer_names: Iterable[str]
                            producer_names, processed_item = result

                            invalid_producers: List[str] = [producer_name for producer_name in producer_names
                                                            if producer_name not in self.producers]
                            if any(invalid_producers):
                                logger.warning(f'Invalid producer: {", ".join(invalid_producers)}')
                                producer_names = [producer_name for producer_name in producer_names
                                                  if producer_name in self.producers]
                            producers = [self.producers[producer_name] for producer_name in producer_names]

                        if not len(producers):
                            logger.warning('Sending an item to an empty set of producers.')

                        if processed_item is not None:
                            self.forward_item(processed_item, producers=producers)

                self.tic()

        except BaseException as e:
            ex = e

        finally:
            self.stop()
            noexcept_exit: Any = self.on_stopped(ex)

            if ex is not None:
                if noexcept_exit is not None and type(noexcept_exit) != bool:
                    logger.warning(f'Return value of `on_stopped` implementation is not bool: {noexcept_exit}. '
                                   f'Raising exception independent of true value of {noexcept_exit}.')

                # If `ex` is not implementing Exception raise either way, independently on whether
                # on_stopped told us to discard the exception
                if not isinstance(ex, BaseException):
                    logger.warning(f'Raised {ex} of type {type(ex)}. Wrapping.')
                    raise Exception(f'{ex}')

                # If on_stopped wants us to raise the exception, we raise.
                if not noexcept_exit:
                    raise ex from None

                # If on_stopped is not implemented or does not override the default, raise whenever the exception
                # is not an instance of StopTimeout. StopTimeouts are only reported.
                if type(noexcept_exit) != bool:
                    if isinstance(ex, StopTimeout):
                        logger.info(f'Node {self.name} stopped ungracefully with a stop timeout.')
                    else:
                        raise ex from None

    def tic(self) -> None:
        """Called periodically in run."""
